// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/12AI64/12AI64_Linux_1.x.x.x_DN/samples/stream/main.c $
// $Rev: 51861 $
// $Date: 2022-10-19 10:15:38 -0500 (Wed, 19 Oct 2022) $

// 12AI64: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _parse_args(int argc, char** argv, args_t* args, tx_data_t* tx, rx_data_t* rx)
{
	gsc_arg_item_t	list[]	=
	{
		//	type				var						arg			values							desc
		{	GSC_ARG_S32_FLAG,	&args->io_mode,			"-bmdma",	{ GSC_IO_MODE_BMDMA },			"Transfer data using Block Mode DMA."						},
		{	GSC_ARG_S32_FLAG,	&args->continuous,		"-c",		{ -1 },							"Continue testing until an error occurs."					},
		{	GSC_ARG_S32_FLAG,	&args->continuous,		"-C",		{ +1 },							"Continue testing even if errors occur."					},
		{	GSC_ARG_S32_MIN,	&args->minute_limit,	"-m",		{ 1 },							"Continue testing for up to # minutes."						},
		{	GSC_ARG_S32_MIN,	&args->test_limit,		"-n",		{ 1 },							"Continue testing for up to # iterations."					},
		{	GSC_ARG_S32_FLAG,	&args->io_mode,			"-pio",		{ GSC_IO_MODE_PIO },			"Transfer data using PIO (default)."						},
		{	GSC_ARG_S32_FLAG,	&rx->option,			"-rx0",		{ RX_OPTION_ZERO_DATA },		"Provide null data rather than read from the device."		},
		{	GSC_ARG_S32_FLAG,	&rx->option,			"-rxr",		{ RX_OPTION_READ_DEV },			"Read data from the device (default)."						},
		{	GSC_ARG_S32_RANGE,	&rx->rx_mb,				"-r",		{ 1, 30000 },					"Recevice # megabytes of data (default: 5)."				},
		{	GSC_ARG_S32_FLAG,	&args->stats,			"-s",		{ 1 },							"Show transfer statistics upon completion."					},
		{	GSC_ARG_S32_RANGE,	&args->fsamp,			"-sr",		{ 1, 500000 },					"Sample input data at # S/S (default: 10,000)."				},
		{	GSC_ARG_S32_FLAG,	&tx->option,			"-tx0",		{ TX_OPTION_BIT_BUCKET },		"Do nothing with the received data (default)."				},
		{	GSC_ARG_S32_FLAG,	&tx->option,			"-txb",		{ TX_OPTION_WRITE_FILE_BIN },	"Write the receive data to a binary file (datast.bin)."		},
		{	GSC_ARG_S32_FLAG,	&tx->option,			"-txt",		{ TX_OPTION_WRITE_FILE_TEXT },	"Write the receive data to a text file (datast.txt)."		},
		{	GSC_ARG_S32_FLAG,	&tx->validate,			"-txv",		{ 1 },							"Enable minor validation and reporting of Tx process data."	},
		{	GSC_ARG_DEV_INDEX,	&args->index,			"index",	{ 1 },							"The zero based index of the device to access."				}
	};

	const gsc_arg_set_t	set	=
	{
		/* name	*/	"stream",
		/* desc	*/	"Stream data from the device to disk.",
		/* qty	*/	SIZEOF_ARRAY(list),
		/* list	*/	list
	};

	int	errs;

	memset(args, 0, sizeof(args_t));
	memset(rx, 0, sizeof(rx_data_t));
	memset(tx, 0, sizeof(tx_data_t));

	args->fsamp		= 10000;
	args->io_mode	= GSC_IO_MODE_PIO;
	rx->rx_mb		= 5;
	rx->option		= RX_OPTION_READ_DEV;
	tx->option		= TX_OPTION_BIT_BUCKET;

	errs	= gsc_args_parse(argc, argv, &set);

	return(errs);
}



//*****************************************************************************
static void _show_access_index(const args_t* args)
{
	gsc_label("Accessing Device");
	printf("%d\n", args->index);
}



//*****************************************************************************
static void _show_time_stamp(void)
{
	const char*	psz;
	struct tm*	stm;
	time_t		tt;

	time(&tt);
	stm	= localtime(&tt);
	psz	= (char*) asctime(stm);
	gsc_label("Time Stamp");
	printf("%s", psz);
}



/******************************************************************************
*
*	Function:	main
*
*	Purpose:
*
*		Control the overall flow of the application.
*
*	Arguments:
*
*		argc			The number of command line arguments.
*
*		argv			The list of command line arguments.
*
*	Returned:
*
*		EXIT_SUCCESS	No errors were encounterred.
*		EXIT_FAILURE	One or more problems were encounterred.
*
******************************************************************************/

int main(int argc, char** argv)
{
	args_t		args;
	int			errs		= 0;
	time_t		exec		= time(NULL);
	long		failures	= 0;
	long		hours;
	long		mins;
	time_t		now;
	long		passes		= 0;
	int			qty			= 0;
	int			ret;
	rx_data_t	rx_data;
	long		secs;
	time_t		t_limit;
	time_t		test;
	long		tests		= 0;
	tx_data_t	tx_data;

	for (;;)
	{
		gsc_label_init(27);
		test	= time(NULL);
		errs	= _parse_args(argc, argv, &args, &tx_data, &rx_data);

		if (errs)
			break;

		// Introduction

		t_limit	= exec + (args.minute_limit * 60);
		os_id_host();
		_show_time_stamp();
		_show_access_index(&args);
		errs	+= ai64_init_util(1);
		errs	+= os_id_driver(ai64_open, ai64_read, ai64_close);
		errs	+= ai64_count_boards(1, &qty);

		if ((errs) || (qty <= 0))
			break;

		// Open access to device

		errs	= ai64_open_util(args.index, 0, -1, 1, &args.fd);

		if (errs == 0)
		{
			// Device identification

			errs	+= ai64_id_device(args.fd, -1, 1);

			// Perform testing

			errs	+= perform_tests(&args, &rx_data, &tx_data);

			// Close device access

			errs	+= ai64_close_util(args.fd, -1, 1);
		}

		// End of processing.

		now	= time(NULL);
		tests++;

		if (errs)
		{
			failures++;
			printf(	"\nRESULTS: FAIL <---  (%d error%s)",
					errs,
					(errs == 1) ? "" : "s");
		}
		else
		{
			passes++;
			printf("\nRESULTS: PASS");
		}

		secs	= (long) (now - test);
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(" (duration %ld:%ld:%02ld)\n", hours, mins, secs);

		secs	= (long) (now - exec);
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(	"SUMMARY: tests %ld, pass %ld, fail %ld"
				" (duration %ld:%ld:%02ld)\n\n",
				tests,
				passes,
				failures,
				hours,
				mins,
				secs);

		if (args.continuous == 0)
			break;

		if ((args.continuous < 0) && (errs))
			break;

		if ((args.minute_limit > 0) && (now >= t_limit))
			break;

		if ((args.test_limit > 0) && (tests >= args.test_limit))
			break;
	}

	ret	= ((qty <= 0) || (errs) || (failures)) ? EXIT_FAILURE : EXIT_SUCCESS;
	return(ret);
}



