// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/12AI64/12AI64_Linux_1.x.x.x_DN/utils/util_id.c $
// $Rev: 51850 $
// $Date: 2022-10-19 10:05:20 -0500 (Wed, 19 Oct 2022) $

// 12AI64: Utilities: source file

#include "main.h"



//*****************************************************************************
static int _id_device_frr(int fd)
{
	int			errs;
	const char*	name;
	u32			val;

	name	= ai64_reg_get_name(AI64_GSC_FRR);
	gsc_label(name);
	errs	= ai64_reg_read(fd, -1, 0, AI64_GSC_FRR, &val);

	if (errs)
		printf("FAIL <---\n");
	else
		printf("0x%08lX\n", (long) val);

	return(errs);
}



//*****************************************************************************
static int _id_device_pci(int fd)
{
	int	errs	= 0;
	u32	reg;

	gsc_label("Vendor ID");
	errs	+= ai64_reg_read(fd, -1, 0, GSC_PCI_9080_VIDR, &reg);
	printf("0x%04lX      ", (long) reg);

	if (reg == 0x10B5)
	{
		printf("(PLX)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	gsc_label("Device ID");
	errs	+= ai64_reg_read(fd, -1, 0, GSC_PCI_9080_DIDR, &reg);
	printf("0x%04lX      ", (long) reg);

	if (reg == 0x9080)
	{
		printf("(PCI9080)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	gsc_label("Sub Vendor ID");
	errs	+= ai64_reg_read(fd, -1, 0, GSC_PCI_9080_SVID, &reg);
	printf("0x%04lX      ", (long) reg);

	if (reg == 0x10B5)
	{
		printf("(PLX)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	gsc_label("Subsystem ID");
	errs	+= ai64_reg_read(fd, -1, 0, GSC_PCI_9080_SID, &reg);
	printf("0x%04lX      ", (long) reg);

	if (reg == 0x2406)
	{
		printf("(12AI64)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	ai64_id_device
*
*	Purpose:
*
*		Identify the device.
*
*	Arguments:
*
*		fd		The handle to use to access the device.
*
*		index	The index of the device or -1 if we don't care.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int ai64_id_device(int fd, int index, int verbose)
{
	s32			device_type;
	const char*	desc	= NULL;
	int			errs;

	if (verbose)
		gsc_label_index("Device", index);

	errs	= ai64_query(fd, -1, 0, AI64_QUERY_DEVICE_TYPE, &device_type);

	if (errs == 0)
	{
		switch (device_type)
		{
			default:

				errs	= 1;

				if (verbose)
				{
					printf(	"FAIL <---  (unexpected device type: %ld)",
							(long) device_type);
				}

				break;

			case GSC_DEV_TYPE_12AI64:
					desc	= "12AI64";
				break;
		}
	}

	if (verbose)
	{
		printf("%s\n", desc ? desc : "");
		gsc_label_level_inc();
	}

	errs	+= _id_device_pci(fd);
	errs	+= _id_device_frr(fd);

	if (verbose)
		gsc_label_level_dec();

	return(errs);
}



