// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/12AI64/12AI64_Linux_1.x.x.x_DN/utils/util_query.c $
// $Rev: 50186 $
// $Date: 2022-02-09 08:27:41 -0600 (Wed, 09 Feb 2022) $

// 12AI64: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ai64_query
*
*	Purpose:
*
*		Provide a visual wrapper for the AI64_IOCTL_QUERY service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the query option to access.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ai64_query(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128]	= "";
	int			errs		= 0;
	const char*	ptr;
	s32			query		= set;
	int			ret;
	s32			tmp;

	if (verbose)
	{
		switch (query)
		{
			default:

				errs++;
				printf(buf, "Query Error (#%ld)", (long) query);
				ptr	= buf;
				break;

			case AI64_QUERY_AUTO_CAL_MS:		ptr	= "Auto-Cal Duration";	break;
			case AI64_QUERY_CHANNEL_MAX:		ptr	= "Max Channels";		break;
			case AI64_QUERY_CHANNEL_QTY:		ptr	= "Installed Channels";	break;
			case AI64_QUERY_COUNT:				ptr	= "Query Options";		break;
			case AI64_QUERY_DEVICE_TYPE:		ptr	= "Device Type";		break;
			case AI64_QUERY_FIFO_SIZE:			ptr	= "FIFO Size";			break;
			case AI64_QUERY_FSAMP_MAX:			ptr	= "Fsamp Max";			break;
			case AI64_QUERY_FSAMP_MIN:			ptr	= "Fsamp Min";			break;
			case AI64_QUERY_INIT_MS:			ptr	= "Init. Duration";		break;
			case AI64_QUERY_MASTER_CLOCK:		ptr	= "Master Clock";		break;
			case AI64_QUERY_NRATE_MAX:			ptr	= "Nrate Max";			break;
			case AI64_QUERY_NRATE_MIN:			ptr	= "Nrate Min";			break;
			case AI64_QUERY_RATE_GEN_QTY:		ptr	= "Rate Gen Qty";		break;
		}

		gsc_label_index(ptr, index);
	}

	ret		= ai64_ioctl(fd, AI64_IOCTL_QUERY, &set);
	errs	+= ret ? 1 : 0;

	if (verbose)
	{
		switch (query)
		{
			default:

				errs++;
				printf(buf, "%ld", (long) set);
				break;

			case AI64_QUERY_CHANNEL_MAX:
			case AI64_QUERY_CHANNEL_QTY:
			case AI64_QUERY_COUNT:
			case AI64_QUERY_NRATE_MAX:
			case AI64_QUERY_NRATE_MIN:

				gsc_label_long_comma_buf(set, buf);
				break;

			case AI64_QUERY_RATE_GEN_QTY:

				gsc_label_long_comma_buf(set, buf);
				sprintf(buf + strlen(buf), " (A-%c)", (int) 'A' - 1 + set);
				break;

			case AI64_QUERY_MASTER_CLOCK:

				gsc_label_long_comma_buf(set, buf);
				strcat(buf, " Hz");
				break;

			case AI64_QUERY_AUTO_CAL_MS:
			case AI64_QUERY_INIT_MS:

				gsc_label_long_comma_buf(set, buf);
				strcat(buf, " ms");
				break;

			case AI64_QUERY_FSAMP_MAX:
			case AI64_QUERY_FSAMP_MIN:

				gsc_label_long_comma_buf(set, buf);
				strcat(buf, " S/S");
				break;

			case AI64_QUERY_FIFO_SIZE:

				ptr	= "";
				tmp	= set;

				if ((tmp) && ((tmp % 1024) == 0))
				{
					ptr	= "K";
					tmp	/= 1024;
				}

				sprintf(buf, "%ld%s Words", (long) tmp, ptr);
				break;

			case AI64_QUERY_DEVICE_TYPE:

				if (set == GSC_DEV_TYPE_12AI64)
				{
					strcpy(buf, "12AI64");
				}
				else
				{
					errs++;
					sprintf(buf, "INVALID: %ld", (long) set);
				}

				break;
		}

		if (errs)
			printf("FAIL <---  (%s)\n", buf);
		else
			printf("%s\n", buf);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


